;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          hyperobject-tests.lisp
;;;; Purpose:       Hyperobject tests file
;;;; Programmer:    Kevin M. Rosenberg
;;;; Date Started:  Apr 2003
;;;;
;;;; $Id: tests.lisp,v 1.4 2003/04/29 09:47:10 kevin Exp $
;;;;
;;;; This file is Copyright (c) 2000-2002 by Kevin M. Rosenberg
;;;; *************************************************************************

(defpackage #:hyperobject-tests
  (:use #:hyperobject #:cl #:rtest #:kmrcl))
(in-package #:hyperobject-tests)

(defclass person (hyperobject)
  ((first-name :initarg :first-name :accessor first-name
	       :value-type (varchar 20)
	       :value-constraint stringp
	       :null-allowed nil)
   (last-name :initarg :last-name :accessor last-name
	      :value-type (varchar 30)
	      :value-constraint stringp
	      :hyperlink find-person-by-last-name
	      :null-allowed nil)
   (full-name :value-type string :stored nil)
   (dob :initarg :dob :accessor dob
	:value-type integer
	:print-formatter date-string
	:value-constraint integerp
	:input-filter convert-to-date)
   (resume :initarg :resume :accessor resume
	   :value-type string
	   :value-constraint stringp)
   ;;   (addresses :value-type (list-of subobject) :initarg :addresses :accessor addresses))
   (addresses :initarg :addresses :accessor addresses
	       :subobject t))
  (:metaclass hyperobject-class)
  (:default-initargs :first-name "" :last-name "" :dob 0 :resume nil) 
  (:default-print-slots first-name last-name dob resume)
  (:user-name "Person")
  (:description "A Person")
  (:direct-rules
   (:rule-1 (:dependants (last-name first-name) :volatile full-name)
	    (setf full-name (concatenate 'string first-name " " last-name)))))

(defclass address (hyperobject)
  ((title :initarg :title :accessor title
	   :value-type (varchar 20)
	   :value-constraint stringp)
   (street :initarg :street :accessor street
	   :value-type (varchar 30)
	   :value-constraint stringp)
   (phones :initarg :phones :accessor phones
	    :subobject t))
  (:metaclass hyperobject-class)
  (:default-initargs :title nil :street nil) 
  (:user-name "Address")
  (:default-print-slots title street)
  (:description "An address"))

(defclass phone (hyperobject)
  ((title :initarg :title :accessor title
	  :value-type (varchar 20)
	  :value-constraint stringp)
   (phone-number :initarg :phone-number :accessor phone-number
		 :value-type (varchar 16)
		 :value-constraint stringp))
  (:metaclass hyperobject-class)
  (:user-name "Phone Number")
  (:default-initargs :title nil :phone-number nil)
  (:default-print-slots title phone-number)
  (:description "A phone number"))

(defparameter home-phone-1 (make-instance 'phone :title "Voice" :phone-number "367-9812"))
(defparameter home-phone-2 (make-instance 'phone :title "Fax" :phone-number "367-9813"))

(defparameter office-phone-1 (make-instance 'phone :title "Main line" :phone-number "123-0001"))
(defparameter office-phone-2 (make-instance 'phone :title "Staff line" :phone-number "123-0002"))
(defparameter office-phone-3 (make-instance 'phone :title "Fax" :phone-number "123-0005"))

(defparameter home (make-instance 'address :title "Home" :street "321 Shady Lane"
				  :phones (list home-phone-1 home-phone-2)))

(defparameter office (make-instance 'address :title "Office" :street "113 Main St."
				    :phones (list office-phone-1 office-phone-2 office-phone-3)))

			      
(defparameter mary (make-instance 'person :first-name "Mary" :last-name "Jackson"
			    :dob (encode-universal-time
				  1 2 3 4 5 2000)
			    :addresses (list home office)
			    :resume "Style & Grace"))


(defun view-to-string (obj &rest args)
  (with-output-to-string (strm)
    (apply #'view obj :stream strm args)))

(rem-all-tests)

(deftest p1 (view-to-string mary) " Person:
  Mary Jackson Thu, 4 May 2000 03:02:01 Style & Grace
")

(deftest p2 (view-to-string mary :subobjects t) " Person:
  Mary Jackson Thu, 4 May 2000 03:02:01 Style & Grace
  Addresss:
    Home 321 Shady Lane
    Phone Numbers:
      Voice 367-9812
      Fax 367-9813
    Office 113 Main St.
    Phone Numbers:
      Main line 123-0001
      Staff line 123-0002
      Fax 123-0005
")

(deftest p3 (view-to-string mary :category :compact-text-labels)
  " Person:
  first-name Mary last-name Jackson dob Thu, 4 May 2000 03:02:01 resume Style & Grace
")

(deftest p4 (view-to-string mary :subobjects t :category :html-labels)
  " <p><b>Person:</b></p><div class=\"person\"><ul>
 <li> <span class=\"label\">first-name</span> <span class=\"first-name\">Mary</span> <span class=\"label\">last-name</span> <span class=\"last-name\">Jackson</span> <span class=\"label\">dob</span> <span class=\"dob\">Thu, 4 May 2000 03:02:01</span> <span class=\"label\">resume</span> <span class=\"resume\">Style & Grace</span>
  <p><b>Addresss:</b></p><div class=\"address\"><ul>
  <li> <span class=\"label\">title</span> <span class=\"title\">Home</span> <span class=\"label\">street</span> <span class=\"street\">321 Shady Lane</span>
    <p><b>Phone Numbers:</b></p><div class=\"phone\"><ul>
    <li> <span class=\"label\">title</span> <span class=\"title\">Voice</span> <span class=\"label\">phone-number</span> <span class=\"phone-number\">367-9812</span>
    </li>
    <li> <span class=\"label\">title</span> <span class=\"title\">Fax</span> <span class=\"label\">phone-number</span> <span class=\"phone-number\">367-9813</span>
    </li>
    </ul></div>
  </li>
  <li> <span class=\"label\">title</span> <span class=\"title\">Office</span> <span class=\"label\">street</span> <span class=\"street\">113 Main St.</span>
    <p><b>Phone Numbers:</b></p><div class=\"phone\"><ul>
    <li> <span class=\"label\">title</span> <span class=\"title\">Main line</span> <span class=\"label\">phone-number</span> <span class=\"phone-number\">123-0001</span>
    </li>
    <li> <span class=\"label\">title</span> <span class=\"title\">Staff line</span> <span class=\"label\">phone-number</span> <span class=\"phone-number\">123-0002</span>
    </li>
    <li> <span class=\"label\">title</span> <span class=\"title\">Fax</span> <span class=\"label\">phone-number</span> <span class=\"phone-number\">123-0005</span>
    </li>
    </ul></div>
  </li>
  </ul></div>
 </li>
 </ul></div>
")

(deftest p5 (view-to-string mary :subobjects t  :category :xml-link-labels)
  " <personlist><title>Person:</title> 
 <person><label>first-name</label> <first-name>Mary</first-name> <label>last-name</label> <xmllink xlink:type=\"simple\" xlink:href=\"disp-func1?func=FIND-PERSON-BY-LAST-NAME&amp;key=Jackson\">Jackson</xmllink> <label>dob</label> <dob><![CDATA[Thu, 4 May 2000 03:02:01]]></dob> <label>resume</label> <resume>Style & Grace</resume>
  <addresslist><title>Addresss:</title> 
  <address><label>title</label> <title>Home</title> <label>street</label> <street>321 Shady Lane</street>
    <phonelist><title>Phone Numbers:</title> 
    <phone><label>title</label> <title>Voice</title> <label>phone-number</label> <phone-number>367-9812</phone-number>
</phone>
    <phone><label>title</label> <title>Fax</title> <label>phone-number</label> <phone-number>367-9813</phone-number>
</phone>
    </phonelist>
</address>
  <address><label>title</label> <title>Office</title> <label>street</label> <street>113 Main St.</street>
    <phonelist><title>Phone Numbers:</title> 
    <phone><label>title</label> <title>Main line</title> <label>phone-number</label> <phone-number>123-0001</phone-number>
</phone>
    <phone><label>title</label> <title>Staff line</title> <label>phone-number</label> <phone-number>123-0002</phone-number>
</phone>
    <phone><label>title</label> <title>Fax</title> <label>phone-number</label> <phone-number>123-0005</phone-number>
</phone>
    </phonelist>
</address>
  </addresslist>
</person>
 </personlist>
")
